import Stream from 'stream';
import { SocketService } from '../SocketService.js';
class DockerLogger {
    getStreamKey(params) {
        const { serverName, containerName, } = params;
        return `${serverName}_${containerName}`;
    }
    getState(params) {
        return DockerLogger.state[this.getStreamKey(params)];
    }
    addStream(params) {
        const streamKey = this.getStreamKey(params);
        if (DockerLogger.state[streamKey]) {
            return {
                message: 'already_exists',
                stream: DockerLogger.state[streamKey],
            };
        }
        DockerLogger.state[streamKey] = {
            nativeStream: new Stream.PassThrough(),
            dockerStream: null,
        };
        return {
            message: 'created',
            state: DockerLogger.state[streamKey],
        };
    }
    listen(container, logParams) {
        const { state, message } = this.addStream(logParams);
        const { containerName, serverName } = logParams;
        if (!state)
            return;
        if (message === 'already_exists')
            return;
        state.nativeStream.on('data', (chunk) => {
            SocketService.emit({ context: `${serverName}_${containerName}`, event: 'log' }, {
                ...logParams,
                log: chunk.toString('utf8'),
            });
        });
        container.logs({
            follow: true,
            stdout: true,
            stderr: true,
            since: Math.floor(new Date().getTime() / 1000),
        }, (err, dockerStream) => {
            if (err)
                return console.error(err);
            state.dockerStream = dockerStream;
            container.modem.demuxStream(dockerStream, state.nativeStream, state.nativeStream);
            dockerStream.on('end', () => {
                state.nativeStream.end('!stop!');
                state.nativeStream.destroy();
                state.dockerStream.destroy();
                this.removeListener(logParams);
            });
        });
    }
    removeListener(logParams) {
        const state = this.getState(logParams);
        state.nativeStream.destroy();
        state.dockerStream.destroy();
        delete DockerLogger.state[this.getStreamKey(logParams)];
    }
}
DockerLogger.state = {};
const instance = new DockerLogger();
export { instance as DockerLogger };
export default instance;
